DEFINE VARIABLE giTotRec AS INTEGER   NO-UNDO.
DEFINE VARIABLE giTotTbl AS INTEGER   NO-UNDO.
DEFINE VARIABLE glAbort  AS LOGICAL   NO-UNDO.
DEFINE VARIABLE i        AS INTEGER   NO-UNDO.
DEFINE TEMP-TABLE tt NO-UNDO
   FIELD _table   AS CHARACTER
   FIELD _field   AS CHARACTER
   FIELD _rowId   AS CHARACTER
   FIELD _label   AS CHARACTER
   FIELD _format  AS CHARACTER
   FIELD _xref    AS LOGICAL
   FIELD _old     AS CHARACTER
   FIELD _new     AS CHARACTER
   INDEX idx1 _rowId _table.
DEFINE TEMP-TABLE ttPort NO-UNDO
   FIELD _old         AS CHARACTER
   FIELD _new         AS CHARACTER
   FIELD _unlo        AS CHARACTER
   FIELD _exist       AS LOGICAL
   FIELD _existActive AS LOGICAL
   FIELD _valid       AS LOGICAL
   FIELD _convert     AS LOGICAL
   FIELD _rowId       AS CHARACTER
   INDEX idx1 _old.
     
/* [MAIN BLOCK> ======================================*/
SESSION:SET-WAIT-STATE('GENERAL').
RUN _loadPorts.
RUN _loadFields.
RUN _scanDb.
/* RUN _updateDb. */
SESSION:SET-WAIT-STATE('').
MESSAGE 'done' 
   SKIP 'log files in directory:' SESSION:TEMP-DIRECTORY 
   VIEW-AS ALERT-BOX INFORMATION.
/*====================================== <MAIN BLOCK] */

FUNCTION writeLog RETURNS LOGICAL 
   (INPUT phBf   AS HANDLE,
    INPUT pcValue AS CHARACTER ):
   DEFINE VARIABLE cFile  AS CHARACTER NO-UNDO.
   DEFINE VARIABLE cLine  AS CHARACTER NO-UNDO.
   DEFINE VARIABLE i      AS INTEGER   NO-UNDO.
   DEFINE VARIABLE cBfVal AS CHARACTER NO-UNDO.
   
   cFile = SESSION:TEMP-DIRECTORY + 'convert' + ENTRY(1,PROGRAM-NAME(2),' ') +  '.txt'.
   
   CASE pcValue:
      WHEN 'CLEAR' THEN 
         DO:
            IF SEARCH(cFile) NE ? THEN
               OS-DELETE VALUE(cFile).
            RETURN YES.      
         END. /* clear */
      WHEN 'LABEL' THEN 
         DO i = 1 TO phBf:NUM-FIELDS:
            cLine = cLine 
                  + FILL(' ',MIN(i - 1,1)) 
                  + phBf:BUFFER-FIELD(i):LABEL.
         END. /* label */
      WHEN 'VALUE' THEN 
         DO i = 1 TO phBf:NUM-FIELDS:
            cBfVal = phBf:BUFFER-FIELD(i):BUFFER-VALUE.
            IF cBfVal EQ ? THEN
               cBfVal = ''.
            IF phBf:BUFFER-FIELD(i):DATA-TYPE EQ 'CHARACTER' THEN 
               cBfVal = '"' + cBfVal + '"' .
            cLine = cLine 
                  + FILL(' ',MIN(i - 1,1))
                  + cBfVal.
         END. /* value */
      OTHERWISE cLine = pcValue.
   END CASE.
   
   IF VALID-HANDLE(phBf) THEN
      cLine = cLine + CHR(10).
   
   OUTPUT TO VALUE(cFile) APPEND.
      PUT UNFORMATTED cLine.
   OUTPUT CLOSE.   
   
   RETURN YES.
END FUNCTION.

PROCEDURE _loadPorts:  
   DEFINE VARIABLE iIncomplete AS INTEGER   NO-UNDO.
   DEFINE BUFFER bPort FOR aa-port.
   
   writeLog(?,'clear').
   writeLog(BUFFER ttPort:HANDLE,'LABEL').
   writeLog(?,CHR(10)).
   
   FOR EACH aa-port NO-LOCK WHERE 
            aa-port.countrycode EQ ? OR
            aa-port.countrycode EQ '':
      CREATE ttPort.
      ASSIGN ttPort._old   = aa-port.portcode
             ttPort._new   = 'empty country cd'
             ttPort._unlo  = aa-port.unlocode
             ttPort._valid = NO
             ttPort._rowId = STRING(ROWID(aa-port))
             iIncomplete   = iIncomplete + 1 .      
      writeLog(BUFFER ttPort:HANDLE,'VALUE').       
   END. /* aa-port */
   FOR EACH aa-port NO-LOCK WHERE 
        NOT aa-port.portcode BEGINS aa-port.countrycode AND
            aa-port.countrycode NE ? AND
            aa-port.countrycode NE '':
      CREATE ttPort.
      ASSIGN ttPort._old   = aa-port.portcode
             ttPort._new   = aa-port.unlocode
             ttPort._unlo  = aa-port.unlocode
             ttPort._rowId = STRING(ROWID(aa-port)).
      IF ttPort._new EQ ?
      OR ttPort._new EQ '' THEN 
         ttPort._new = aa-port.countrycode + aa-port.portcode.
      FOR EACH bPort NO-LOCK WHERE 
               bPort.portcode EQ ttPort._new:
         ttPort._exist       = YES.
         ttPort._existActive = NOT bPort.ind_inactive 
                               OR ttPort._existAct.
      END. /* bPort */      
      ttPort._convert = ttPort._new BEGINS aa-port.countryCode
                      AND aa-port.countrycode NE ?
                      AND aa-port.countrycode NE ''
                      AND LENGTH(ttPort._new) EQ 5
                      AND NOT ttPort._exist.
      ttPort._valid = ttPort._convert
                      OR (ttPort._new BEGINS aa-port.countryCode
                          AND aa-port.countrycode NE ?
                          AND aa-port.countrycode NE ''
                          AND LENGTH(ttPort._new) EQ 5
                          AND ttPort._exist).
      writeLog(BUFFER ttPort:HANDLE,'VALUE').
      IF NOT ttPort._valid THEN
         iIncomplete = iIncomplete + 1.
   END. /* aa-port */
   IF iIncomplete NE 0 THEN DO:
      glAbort = YES.
      MESSAGE iIncomplete 'incomplete port records found.'
         SKIP 'Abort?'
         VIEW-AS ALERT-BOX QUESTION BUTTONS YES-NO UPDATE glAbort.
   END.
END PROCEDURE.

PROCEDURE _LoadFields:
   DEFINE VARIABLE iDb      AS INTEGER   NO-UNDO.
   DEFINE VARIABLE hBfField AS HANDLE    NO-UNDO.
   DEFINE VARIABLE hBfFile  AS HANDLE    NO-UNDO.
   DEFINE VARIABLE hQry     AS HANDLE    NO-UNDO.
   DEFINE VARIABLE iFld     AS INTEGER   NO-UNDO.
   DEFINE VARIABLE cTbl     AS CHARACTER NO-UNDO.
   DEFINE VARIABLE cQryStr  AS CHARACTER NO-UNDO.
   DEFINE VARIABLE cDbLst   AS CHARACTER NO-UNDO.
   DEFINE VARIABLE cMsg     AS CHARACTER NO-UNDO.

   /*-- method query-prepare does not like combined strings --*/
   cQryStr = 'FOR EACH _field WHERE '
           +    '(   INDEX(_field._field-name,"portcode") NE 0 '
           +     'OR INDEX(_field._field-name,"city_receipt") NE 0 '
           +     'OR INDEX(_field._field-name,"citycode") NE 0 ) AND '
           +    'INDEX(_field._field-name,"report") EQ 0 AND '
           +    'INDEX(_field._field-name,"rapport") EQ 0 AND '
           +    'INDEX(_field._field-name,"transport") EQ 0 AND '
/*            +    'INDEX(_field._field-name,"import") EQ 0 AND '    */
           +    'INDEX(_field._field-name,"export") EQ 0 AND '
/*            +    'INDEX(_field._field-name,"portbase") EQ 0 AND '  */
           +    'INDEX(_field._field-name,"airport") EQ 0 AND '
/*            +    'INDEX(_field._field-name,"ftp") EQ 0 AND '       */
/*            +    'INDEX(_field._field-name,"printer") EQ 0 AND '   */
           +      'NOT _field._Field-name BEGINS "_" AND '
           +          '_field._data-type EQ "CHARACTER", '
           + 'FIRST _file WHERE '
           + 'RECID(_file)             EQ _field._File-recid AND '
           +       '_file._file-name   NE "aa-port" AND ' /* process seperatly */
           +       '_file._Hidden      EQ NO AND ' 
           +       '_file._File-Number GT 0 AND '
           +       '_file._File-Number LT 1000'.

   writeLog(?,'clear').
   writeLog(BUFFER tt:HANDLE,'LABEL').
   writeLog(?,CHR(10)).
   
   DO iDb = 1 TO NUM-DBS:
      cDbLst = cDbLst + MINIMUM(cDbLst,',') + LDBNAME(iDb).
      CREATE BUFFER hBfFile  FOR TABLE LDBNAME(iDb) + '._file'.
      CREATE BUFFER hBfField FOR TABLE LDBNAME(iDb) + '._field'.
      CREATE QUERY hQry.  
      hQry:SET-BUFFERS(hBfField,hBfFile).
      hQry:QUERY-PREPARE(cQryStr).
      hQry:QUERY-OPEN().      
      hQry:GET-FIRST(NO-LOCK).
      DO WHILE hBfField:AVAILABLE:
         PROCESS EVENTS.
         CREATE tt.
         ASSIGN tt._table  = hBfFile :BUFFER-FIELD('_file-name') :BUFFER-VALUE
                tt._field  = hBfField:BUFFER-FIELD('_field-name'):BUFFER-VALUE
                tt._label  = hBfField:BUFFER-FIELD('_label')     :BUFFER-VALUE
                tt._format = hBfField:BUFFER-FIELD('_format')    :BUFFER-VALUE
                tt._rowId  = ? /* to be picked up by _scanDb as table master record */
                tt._xref   = CAN-FIND(FIRST sp-crossref NO-LOCK WHERE 
                                            sp-crossref.rel_tabel EQ 'aa-port' AND
                                            sp-crossref.rel_veld  EQ 'portcode' AND
                                            sp-crossref.tabel     EQ tt._table AND
                                            sp-crossref.veldnaam  EQ tt._field).
         IF cTbl NE tt._table THEN ASSIGN
            giTotTbl = giTotTbl + 1
            cTbl     = tt._table.
         writeLog(BUFFER tt:HANDLE,'VALUE').
         hQry:GET-NEXT(NO-LOCK).
      END. /*each file*/
      hQry:QUERY-CLOSE().
      DELETE OBJECT hBfFile.   
      DELETE OBJECT hBfField.   
      DELETE OBJECT hQry.   
   END. /* num-dbs */
   FOR EACH sp-crossref NO-LOCK WHERE 
            sp-crossref.rel_tabel EQ 'aa-port' AND
            sp-crossref.rel_veld  EQ 'portcode' AND
            CAN-DO(cDbLst,SUBSTRING(sp-crossref.tabel,1,2)) AND 
            NOT CAN-FIND(tt WHERE 
                         tt._table EQ sp-crossref.tabel AND 
                         tt._field EQ sp-crossref.veldnaam):
      cMsg = cMsg + CHR(10) + sp-crossref.tabel + '.' + sp-crossref.veldnaam.
   END.
   FOR EACH sp-crossref NO-LOCK WHERE 
            sp-crossref.vrij MATCHES '*aa-port*' AND
            NOT CAN-FIND(tt WHERE 
                         tt._table EQ sp-crossref.tabel AND 
                         tt._field EQ sp-crossref.veldnaam) :
      cMsg = cMsg + CHR(10) + sp-crossref.tabel + '.' + sp-crossref.veldnaam + ' (vrije ingave)'.
   END.
   IF cMsg NE '' THEN
      MESSAGE 'missing crossref fields:' cMsg VIEW-AS ALERT-BOX.
END PROCEDURE.
 
PROCEDURE _scanDb:
   DEFINE VARIABLE hQry    AS HANDLE    NO-UNDO.
   DEFINE VARIABLE hBf     AS HANDLE    NO-UNDO.
   DEFINE VARIABLE hFld    AS HANDLE    NO-UNDO.
   DEFINE VARIABLE iNumRec AS INTEGER   NO-UNDO.
   DEFINE VARIABLE dPerc   AS DECIMAL   NO-UNDO.
   DEFINE VARIABLE cDes    AS CHARACTER NO-UNDO.
   DEFINE BUFFER ttChild FOR tt.
   DEFINE BUFFER ttTable FOR tt.
   DEFINE BUFFER ttField FOR tt.
   
   FORM cDes     AT 1 FORMAT 'X(30)' NO-LABEL
        dPerc         FORMAT '>>9 %' NO-LABEL SKIP
        giTotRec AT 1                   LABEL 'num records'
     WITH FRAME frm-scan 1 DOWN SIDE-LABELS VIEW-AS DIALOG-BOX.

   IF glAbort THEN
      RETURN.
   
   FOR EACH ttTable EXCLUSIVE-LOCK WHERE
            ttTable._rowId EQ ?:
      iNumRec = iNumRec + 1.
      DISPLAY
         ttTable._table @ cDes
         ((iNumRec / giTotTbl) * 100) @ dPerc
         giTotRec
         WITH FRAME frm-scan.
      PAUSE 0.
      CREATE BUFFER hBf FOR TABLE ttTable._table.
      CREATE QUERY hQry.  
      hQry:SET-BUFFERS(hBf).
      /*-- for all companies, since aa-port is for all companies --*/
      hQry:QUERY-PREPARE('FOR EACH ' + ttTable._table).
      hQry:QUERY-OPEN().      
      hQry:GET-FIRST(NO-LOCK).
      DO WHILE hBf:AVAILABLE:
         PROCESS EVENTS.
         IF giTotRec MODULO 100 EQ 0 THEN DO:
            DISPLAY
               ttTable._table @ cDes
               ((iNumRec / giTotTbl) * 100) @ dPerc
               giTotRec
               WITH FRAME frm-scan.
            PAUSE 0.   
         END.   
         fld:
         FOR EACH ttField EXCLUSIVE-LOCK WHERE 
                  ttField._table EQ ttTable._table AND
                  ttField._rowId EQ ? :
            PROCESS EVENTS.
            IF giTotRec MODULO 100 EQ 0 THEN DO:
               DISPLAY
                  ttTable._table @ cDes
                  ((iNumRec / giTotTbl) * 100) @ dPerc
                  giTotRec
                  WITH FRAME frm-scan.
               PAUSE 0.   
            END.
            
            hFld = hBf:BUFFER-FIELD(ttField._field).
            
            FIND ttPort NO-LOCK WHERE ttPort._old EQ hFld:BUFFER-VALUE NO-ERROR.
            
            IF NOT AVAILABLE ttPort THEN
               NEXT fld.
               
            CREATE ttChild.
            BUFFER-COPY ttField 
                     TO ttChild
                 ASSIGN ttChild._rowid = STRING(hBf:ROWID)
                        ttChild._old   = hFld:BUFFER-VALUE
                        ttChild._new   = ttPort._new
                        giTotRec       = giTotRec + 1.
         END. /* ttField */
         hQry:GET-NEXT(NO-LOCK).
      END. /* hBf available */
      hQry:QUERY-CLOSE().
      DELETE OBJECT hBf.
      DELETE OBJECT hQry.   
      
      FOR EACH ttField EXCLUSIVE-LOCK WHERE 
               ttField._table EQ ttTable._table AND
               ttField._field NE ttTable._field AND
               ttField._rowId EQ ? :
         DELETE ttField.      
      END. /* ttField */
      DELETE ttTable.
   END. /* ttTable */
   
   /*-- for performance write to disk outside main loop --*/
   writeLog(?,'clear').
   writeLog(BUFFER tt:HANDLE,'LABEL').
   writeLog(?,CHR(10)).
   iNumRec = 0.
   FOR EACH tt NO-LOCK:
      iNumRec = iNumRec + 1.
      PROCESS EVENTS.
      IF iNumRec MODULO 100 EQ 0 THEN DO:
         DISPLAY
            'Writing to log' @ cDes
            ((iNumRec / giTotRec) * 100) @ dPerc
            (giTotRec - iNumRec) @ giTotRec
            WITH FRAME frm-scan.
         PAUSE 0.   
      END.   
      writeLog(BUFFER tt:HANDLE,'VALUE').
   END.
   
END PROCEDURE.

PROCEDURE _updateDb:
   DEFINE VARIABLE hBf     AS HANDLE    NO-UNDO.
   DEFINE VARIABLE iFld    AS INTEGER   NO-UNDO.
   DEFINE VARIABLE iNumRec AS INTEGER   NO-UNDO.
   DEFINE VARIABLE rRowId  AS ROWID     NO-UNDO.
   DEFINE VARIABLE lUpdate AS LOGICAL   NO-UNDO INITIAL NO.
   
   IF glAbort THEN
      RETURN.
   
   FOR EACH tt NO-LOCK
            BREAK BY tt._table 
                  BY tt._rowId:
      PROCESS EVENTS.
      iNumRec = iNumRec + 1.
      IF iNumRec MODULO 100 EQ 1 THEN DO:
         DISPLAY
            STRING((iNumRec / giTotRec) * 100,'>>9 %') NO-LABEL
            WITH FRAME frm-update 1 DOWN SIDE-LABELS VIEW-AS DIALOG-BOX.
         PAUSE 0.
      END.
      
      /*-- new buffer --*/
      IF NOT VALID-HANDLE(hBf) 
      OR hBf:TABLE NE tt._table THEN DO:
         IF VALID-HANDLE(hBf) THEN DO:
            hBf:BUFFER-RELEASE().
            DELETE OBJECT hBf.      
         END. /* valid hBf */
         CREATE BUFFER hBf FOR TABLE tt._table.
         hBf:DISABLE-LOAD-TRIGGERS(NO).
      END. /* new buffer */
      
      /*-- new rowId --*/         
      rRowId = TO-ROWID(tt._rowId).
      IF NOT hBf:AVAILABLE 
      OR hBf:ROWID NE rRowId THEN DO:
         IF hBf:AVAILABLE THEN
            hBf:BUFFER-RELEASE().
         hBf:FIND-BY-ROWID(rRowId,EXCLUSIVE-LOCK).
      END. /* new rowId */
      
      IF lUpdate THEN    
         hBf:BUFFER-FIELD(tt._field):BUFFER-VALUE = tt._new. 
      
      IF LAST(tt._table) THEN DO:
         hBf:BUFFER-RELEASE().
         DELETE OBJECT hBf.
      END.
   END. /* tt */
   IF lUpdate THEN DO:
      DISABLE TRIGGERS FOR LOAD OF aa-port.
      FOR EACH ttPort NO-LOCK WHERE ttPort._convert:
         rRowId = TO-ROWID(ttPort._rowId).
         FOR EACH aa-port EXCLUSIVE-LOCK WHERE ROWID(aa-port) EQ rRowId:
            aa-port.portcode EQ ttPort._new.      
         END. /* aa-port */
      END. /* ttPort */
   END. /* lUpdate */
      
END PROCEDURE.

